"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

Group badges
"""
from typing import Any
from typing import Dict

import dict_tools.differ as differ


async def present(
    hub, ctx, name: str, project_id: int, link_url: str, image_url: str
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Adds a badge to a group.


    Args:
        name(Text): The identifier for this state.
        project_id(int): The ID or URL-encoded path of the group owned by the authenticated user.
        link_url(str): URL of the badge link.
        image_url(str): URL of the badge image.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              gitlab.group.group_badges.present:
                - name: value
                - project_id: value
                - link_url: value
                - image_url: value
    """

    result = dict(comment="", changes=None, name=name, result=True)

    before = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/badges",
        data={"kwargs": '{"name": name}'},
        success_codes=[200],
    )
    if before["status"]:
        result["comment"] = f"'{name}' already exists"
    else:
        ret = await hub.exec.request.json.post(
            ctx,
            success_codes=[201, 304, 204],
            url=f"{ctx.acct.endpoint_url}/groups/{project_id}/badges",
            **{"link_url": link_url, "image_url": image_url},
        )
        result["result"] = ret["status"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = f"Created '{name}'"

    # Now that the resource exists, update it
    ret = await hub.exec.request.json.put(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/badges",
        success_codes=[200, 204, 304],
    )

    if not ret["status"]:
        result["status"] = False
        result["comment"] = f"Unable to update '{name}': {ret['comment']}"

    after = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/badges",
        data={"kwargs": '{"name": name}'},
        success_codes=[200],
    )
    result["changes"] = differ.deep_diff(before["ret"], after["ret"])
    return result


async def absent(hub, ctx, name: str, project_id: int, badge_id: int) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Adds a badge to a group.


    Args:
        name(Text): The identifier for this state.
        project_id(int): The ID or URL-encoded path of the group owned by the authenticated user.
        badge_id(int): The badge ID.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              gitlab.group.group_badges.absent:
                - name: value
                - project_id: value
                - badge_id: value
    """

    result = dict(comment="", changes=None, name=name, result=True)
    before = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/badges",
        data={"kwargs": '{"name": name}'},
        success_codes=[204, 304, 404],
    )

    if before["status"]:
        result["comment"] = f"'{name}' already absent"
    else:
        ret = await hub.exec.request.json.delete(
            ctx,
            url=f"{ctx.acct.endpoint_url}/groups/{project_id}/badges",
            success_code=[204],
            **{"badge_id": badge_id},
        )
        result["result"] = ret["status"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = f"Deleted '{name}'"

    after = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/badges",
        data={"kwargs": '{"name": name}'},
        success_codes=[204, 304, 404],
    )

    result["changes"] = differ.deep_diff(before["ret"], after["ret"])
    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function


    Gets a list of a group’s badges.



    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe gitlab.group.group_badges
    """

    result = {}

    async for project in hub.exec.gitlab.request.paginate(
        ctx, url=f"{ctx.acct.endpoint_url}/groups"
    ):
        project_id = project["id"]

        async for ret in hub.exec.gitlab.request.paginate(
            ctx, url=f"{ctx.acct.endpoint_url}/groups/{project_id}/badges"
        ):
            result[f"/groups-{project_id}-group.group_badges-{ret['id']}"] = {
                "gitlab.group.group_badges.present": [
                    {"project_id": ret.get("id")},
                    {"link_url": ret.get("link_url")},
                    {"image_url": ret.get("image_url")},
                ]
            }

    return result
