"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

Group wikis
"""
from typing import Any
from typing import Dict

import dict_tools.differ as differ


async def present(
    hub, ctx, name: str, project_id: int, file: str, branch: str = None
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Upload a file to the attachment folder inside the wiki’s repository. The
    attachment folder is the uploads folder.


    Args:
        name(Text): The identifier for this state.
        project_id(int): The ID or URL-encoded path of the group.
        file(str): The attachment to be uploaded.
        branch(str, optional): The name of the branch. Defaults to the wiki repository default branch. Defaults to None.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              gitlab.group.group_wikis.present:
                - name: value
                - project_id: value
                - file: value
    """

    result = dict(comment="", changes=None, name=name, result=True)

    before = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/wikis",
        data={},
        success_codes=[200],
    )
    if before["status"]:
        result["comment"] = f"'{name}' already exists"
    else:
        ret = await hub.exec.request.json.post(
            ctx,
            success_codes=[201, 304, 204],
            url=f"{ctx.acct.endpoint_url}/groups/{project_id}/wikis",
            **{"file": file, "branch": branch},
        )
        result["result"] = ret["status"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = f"Created '{name}'"

    # Now that the resource exists, update it
    ret = await hub.exec.request.json.put(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/wikis",
        success_codes=[200, 204, 304],
    )

    if not ret["status"]:
        result["status"] = False
        result["comment"] = f"Unable to update '{name}': {ret['comment']}"

    after = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/wikis",
        data={},
        success_codes=[200],
    )
    result["changes"] = differ.deep_diff(before["ret"], after["ret"])
    return result


async def absent(hub, ctx, name: str, project_id: int, slug: str) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Upload a file to the attachment folder inside the wiki’s repository. The
    attachment folder is the uploads folder.


    Args:
        name(Text): The identifier for this state.
        project_id(int): The ID or URL-encoded path of the group.
        slug(str): URL-encoded slug (a unique string) of the wiki page, such as dir%2Fpage_name.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              gitlab.group.group_wikis.absent:
                - name: value
                - project_id: value
                - slug: value
    """

    result = dict(comment="", changes=None, name=name, result=True)
    before = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/wikis",
        data={},
        success_codes=[204, 304, 404],
    )

    if before["status"]:
        result["comment"] = f"'{name}' already absent"
    else:
        ret = await hub.exec.request.json.delete(
            ctx,
            url=f"{ctx.acct.endpoint_url}/groups/{project_id}/wikis",
            success_code=[204],
            **{"slug": slug},
        )
        result["result"] = ret["status"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = f"Deleted '{name}'"

    after = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/groups/{project_id}/wikis",
        data={},
        success_codes=[204, 304, 404],
    )

    result["changes"] = differ.deep_diff(before["ret"], after["ret"])
    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function


    List all wiki pages for a given group.



    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe gitlab.group.group_wikis
    """

    result = {}

    async for project in hub.exec.gitlab.request.paginate(
        ctx, url=f"{ctx.acct.endpoint_url}/groups"
    ):
        project_id = project["id"]

        async for ret in hub.exec.gitlab.request.paginate(
            ctx, url=f"{ctx.acct.endpoint_url}/groups/{project_id}/wikis"
        ):
            result[f"/groups-{project_id}-group.group_wikis-{ret['id']}"] = {
                "gitlab.group.group_wikis.present": [
                    {"project_id": ret.get("id")},
                    {"file": ret.get("file")},
                    {"branch": ret.get("branch")},
                ]
            }

    return result
