"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

Freeze Periods
"""
from typing import Any
from typing import Dict

import dict_tools.differ as differ


async def present(
    hub,
    ctx,
    name: str,
    project_id: int,
    freeze_start: str,
    freeze_end: str,
    cron_timezone: str = None,
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Create a Freeze Period.


    Args:
        name(Text): The identifier for this state.
        project_id(int): The ID or URL-encoded path of the project.
        freeze_start(str): Start of the Freeze Period in cron format.
        freeze_end(str): End of the Freeze Period in cron format.
        cron_timezone(str, optional): The time zone for the cron fields, defaults to UTC if not provided. Defaults to None.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              gitlab.project.freeze_periods.present:
                - name: value
                - project_id: value
                - freeze_start: value
                - freeze_end: value
    """

    result = dict(comment="", changes=None, name=name, result=True)

    before = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/projects/{project_id}/freeze_periods",
        data={},
        success_codes=[200],
    )
    if before["status"]:
        result["comment"] = f"'{name}' already exists"
    else:
        ret = await hub.exec.request.json.post(
            ctx,
            success_codes=[201, 304, 204],
            url=f"{ctx.acct.endpoint_url}/projects/{project_id}/freeze_periods",
            **{
                "freeze_start": freeze_start,
                "freeze_end": freeze_end,
                "cron_timezone": cron_timezone,
            },
        )
        result["result"] = ret["status"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = f"Created '{name}'"

    # Now that the resource exists, update it
    ret = await hub.exec.request.json.put(
        ctx,
        url=f"{ctx.acct.endpoint_url}/projects/{project_id}/freeze_periods",
        success_codes=[200, 204, 304],
    )

    if not ret["status"]:
        result["status"] = False
        result["comment"] = f"Unable to update '{name}': {ret['comment']}"

    after = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/projects/{project_id}/freeze_periods",
        data={},
        success_codes=[200],
    )
    result["changes"] = differ.deep_diff(before["ret"], after["ret"])
    return result


async def absent(
    hub, ctx, name: str, project_id: int, freeze_period_id: str
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Create a Freeze Period.


    Args:
        name(Text): The identifier for this state.
        project_id(int): The ID or URL-encoded path of the project.
        freeze_period_id(str): The database ID of the Freeze Period.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              gitlab.project.freeze_periods.absent:
                - name: value
                - project_id: value
                - freeze_period_id: value
    """

    result = dict(comment="", changes=None, name=name, result=True)
    before = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/projects/{project_id}/freeze_periods",
        data={},
        success_codes=[204, 304, 404],
    )

    if before["status"]:
        result["comment"] = f"'{name}' already absent"
    else:
        ret = await hub.exec.request.json.delete(
            ctx,
            url=f"{ctx.acct.endpoint_url}/projects/{project_id}/freeze_periods",
            success_code=[204],
            **{"freeze_period_id": freeze_period_id},
        )
        result["result"] = ret["status"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = f"Deleted '{name}'"

    after = hub.exec.request.json.get(
        ctx,
        url=f"{ctx.acct.endpoint_url}/projects/{project_id}/freeze_periods",
        data={},
        success_codes=[204, 304, 404],
    )

    result["changes"] = differ.deep_diff(before["ret"], after["ret"])
    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function


    Paginated list of Freeze Periods, sorted by created_at in ascending order.



    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe gitlab.project.freeze_periods
    """

    result = {}

    async for project in hub.exec.gitlab.request.paginate(
        ctx, url=f"{ctx.acct.endpoint_url}/projects"
    ):
        project_id = project["id"]

        async for ret in hub.exec.gitlab.request.paginate(
            ctx, url=f"{ctx.acct.endpoint_url}/projects/{project_id}/freeze_periods"
        ):
            result[f"/projects-{project_id}-project.freeze_periods-{ret['id']}"] = {
                "gitlab.project.freeze_periods.present": [
                    {"project_id": ret.get("id")},
                    {"freeze_start": ret.get("freeze_start")},
                    {"freeze_end": ret.get("freeze_end")},
                    {"cron_timezone": ret.get("cron_timezone")},
                ]
            }

    return result
